<?php

function htmlescape($str)
{
	return htmlspecialchars($str, ENT_QUOTES, 'UTF-8');
}


function parseCond(&$data, $key)
{
	if (isset($data[$key]))
	{
		$var = $data[$key];
		return  (is_array($var) && count($var)) ||
	    	   (!is_array($var) && $var);
	}
	else
		return FALSE;
}

function parseTemplate(&$data, $file)
{
	// template is NOT php-script, read it as plain text
	$text = file_get_contents($file);

	// then if data provided, substitute placeholders
	if (is_array($data) && count($data))
	{
		// split to parts divided by "{ key }"
		$parts = preg_split('/(\{.*\})/U', $text, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
		// ... if any, process substitutions
		if (count($parts) > 1)
		{
			$block = array('skip' => false, 'kind' => 'main');
			$blocks = array();

			$text = '';
			$i = 0;
			while ($i < count($parts))
			{
				$clause = $parts[$i++];

				if ($clause{0} == '{')
				{
					$key = trim(substr($clause, 1, -1)); // cut "{" & "}"

					if ($key{0} == '%')
					{
						if ($key == '%END')
						{
							if ($block['kind'] == 'foreach' && !$block['skip'])
							{
								$arr =& $data[$block['var']];
								if(list(, $data[$block['item']]) = each($arr))
									$i = $block['line'];
								else
									$block = array_pop($blocks);

							}
							else
								$block = array_pop($blocks);
                        }
						else if (substr($key, 0, 3) == '%IF')
						{
							array_push($blocks, $block);

							$key = trim(substr($key, 3));
							$skip = $block['skip'] || !isset($data[$key]) || !parseCond($data, $key);
							$block = array('skip' => $skip, 'kind' => 'if', 'var' => $key);
						}
						else if ($key == '%ELSE')
						{
							$block['skip'] = $blocks[count($blocks)-1]['skip'] || parseCond($data, $block['var']);
						}
						else if (substr($key, 0, 8) == '%FOREACH')
						{
							array_push($blocks, $block);

							list($arrname, $itemname) = array_map('trim', explode('AS', substr($key, 8)));
							$skip = $block['skip'] || !isset($data[$arrname]);
							if (!$skip)
							{
								$arr =& $data[$arrname];
								reset($arr);
								$skip = (list(, $data[$itemname]) = each($arr)) === FALSE;
							}
							$block = array('skip' => $skip, 'kind' => 'foreach', 'var' => $arrname, 'item' => $itemname, 'line' => $i);
						}
						// recursively parse sub-templates
						else if (substr($key, 0, 8) == '%INCLUDE' && !$block['skip'])
						{
							$incFile = dirname($file) . '/' . trim(substr($key, 8)) . '.tpl';
							if (file_exists($incFile))
								$text .= parseTemplate($data, $incFile);
						}

						continue;
					}
					else
					{
						$escape = FALSE;
						$success = TRUE;

						// it is wise to escape user data
						if ($key{0} == '@')
						{
							$key = substr($key, 1);
							$escape = TRUE;
						}

						// (multidimentional) array access
						if (strpos($key, ':') !== FALSE)
						{
							$keys = explode(':', $key);
							$d =& $data;
							foreach ($keys as $key)
							{
								if (isset($d[$key]))
									$d = &$d[$key];	// iterate like chained list
								else
								{
									$success = FALSE;
									break;
								}
							}
							if ($success)
								$clause = $d;
						}
						// simple var
						else
						{
							if (isset($data[$key]))
								$clause = $data[$key];
							else
								$success = FALSE;
						}

						// was '@' modificator
						if ($success && $escape)
							$clause = htmlescape($clause);
					}
				}

				if (!$block['skip'])
					$text .= $clause;
			}
		}
	}

	return $text;
}
